import subprocess
from prompt_toolkit.styles import Style
from pathlib import Path
from socket import gethostname
import os
import getpass

style = Style.from_dict({
    'username': '#0FFFFF bold',
    'dir': '#FFFFFF bold',
    'mid': '#0FFFFF bold',
    'parantheses1': '#FFFFFF bold',
    'git-branch': '#FF0000 bold',
    'star' : '#FFFF00 bold',
    'parantheses2': '#FFFFFF bold',
    'end': "#0FFFFF bold"
})

HOME_DIR = str(Path.home())
hostname = gethostname()
history = Path(HOME_DIR + "/.yash_history")

x = subprocess.getoutput("git rev-parse --is-inside-work-tree")

part1 =""
part2 =""
part3 =""

if str(x) == "true":
    changed_files = subprocess.getoutput("git ls-files -m")
    added_files = subprocess.getoutput("git diff --name-only --cached")
    current_branch = subprocess.getoutput("git branch --show-current")
    if len(changed_files) != 0 or len(added_files) != 0:
        part1 = "[" + str(getpass.getuser()) + "@" + hostname + " "
        part2 = str(os.path.basename(str(os.getcwd())))
        part3 = "]"
        part4 = "("
        part5 = current_branch + " "
        part6 = "*"
        part7 = ")"
        part8 = "$ "
    else:
        part1 = "[" + str(getpass.getuser()) + "@" + hostname + " "
        part2 = str(os.path.basename(str(os.getcwd())))
        part3 = "]"
        part4 = "("
        part5 = current_branch + " "
        part6 = "✓"
        part7 = ")"
        part8 = "$ "                   
else:
    part1 = "[" + str(getpass.getuser()) + "@" + hostname + " "
    part2 = str(os.path.basename(str(os.getcwd())))
    part3 = "]$ "

input_required = [
    ('class:username', part1),
    ('class:dir', part2),
    ('class:mid', part3),
    ('class:parantheses1', part4),
    ('class:git-branch', part5),
    ('class:star', part6),
    ('class:parantheses', part7),
    ('class:end', part8)
]




import os
import subprocess
from prompt_toolkit.styles import Style
from pathlib import Path
from socket import gethostname
import getpass
import shutil
from datetime import datetime

style = Style.from_dict({
    'parantheses1': '#FFFFFF bold',
    'username': '#FF0000 bold',
    'parantheses2': '#FFFFFF bold',
    'parantheses3': '#FFFFFF bold',
    'hostname': '#00FF00 bold',
    'pipe': '#FFFFFF bold',
    'date-time': '#0FFFFF bold',
    'parantheses4': '#FFFFFF bold',
    'parantheses5': '#FFFFFF bold',
    'git-branch': '#0000FF bold',
    'star' : '#FFFF00 bold',
    'parantheses6': '#FFFFFF bold',
    'end': "#FFFF00 bold"
})

HOME_DIR = str(Path.home())
hostname = str(gethostname())
history = Path(HOME_DIR + "/.yash_history")

x = subprocess.getoutput("git rev-parse --is-inside-work-tree")

rightNow = datetime.now()
now = rightNow.strftime("%d/%m/%Y %H:%M:%S")

part1=""
part2=""
part3=""
part4=""
part5=""
part6=""
part7=""
part8=""
part9=""
part10=""
part11=""
part12=""
part13=""

if str(x) == "true":
    changed_files = subprocess.getoutput("git ls-files -m")
    added_files = subprocess.getoutput("git diff --name-only --cached")
    current_branch = subprocess.getoutput("git branch --show-current")
    noOfColumns = shutil.get_terminal_size()[0]
    if len(changed_files) != 0 or len(added_files) != 0:
        part1 = "(" 
        part2 = str(getpass.getuser()) + "@" + str(os.path.basename(os.getcwd()))
        part3 = ")"
        part5 = hostname
        part6 =" | "
        part7 = now
        part8 = ")" + "\n"
        new_str = part1 + part2 + part3 + part5 + part6 + part7 + part8
        width = noOfColumns - len(new_str) -1
        part4 = "(".rjust(width, "-")
        part9 = "("
        part10 = current_branch + " "
        part11 = "*"
        part12 = ")"
        part13 = "$ "
    else:        
        part1 = "(" 
        part2 = str(getpass.getuser()) + "@" + str(os.path.basename(os.getcwd()))
        part3 = ")"
        part5 = hostname
        part6 =" | "
        part7 = now
        part8 = ")" + "\n"
        new_str = part1 + part2 + part3 + part5 + part6 + part7 + part8
        width = noOfColumns - len(new_str) -1
        part4 = "(".rjust(width, "-")
        part9 = "("
        part10 = current_branch + " "
        part11 = "✔"
        part12 = ")"
        part13 = "$ "                 
else:    
    noOfColumns = shutil.get_terminal_size()[0]
    part1 = "(" 
    part2 = str(getpass.getuser()) + "@" + str(os.path.basename(os.getcwd()))
    part3 = ")"
    part5 = hostname
    part6 =" | "
    part7 = now
    part8 = ")" + "\n"
    new_str = part1 + part2 + part3 + part5 + part6 + part7 + part8
    width = noOfColumns - len(new_str) -1
    part4 = "(".rjust(width, "-")
    part13 = "$ "

input_required = [
    ('class:parantheses1', part1),
    ('class:username', part2),
    ('class:parantheses2', part3),
    ('class:parantheses3', part4),
    ('class:hostname', part5),
    ('class:pipe', part6),
    ('class:date-time', part7),
    ('class:parantheses4', part8),
    ('class:parantheses5', part9),
    ('class:git-branch', part10),
    ('class:star', part11),
    ('class:parantheses6', part12),
    ('class:end', part13)
]




from pathlib import Path
from shutil import copyfile
from os.path import isdir, expanduser, dirname, realpath
from distutils.dir_util import copy_tree
from os import name

home_dir = expanduser("~")
config_directory = home_dir + "/.config/"
yash_directory =config_directory+ "/neosh"
yaml_directory = yash_directory + "/config.yaml"
file_location = dirname(realpath(__file__))


def checker():
    Path(config_directory).mkdir(parents=True, exist_ok=True)
    Path(yash_directory).mkdir(parents=True, exist_ok=True)
    if Path(yaml_directory).exists() != True:
        copyfile(file_location + "/neosh/config.yaml", yaml_directory)
    themes_directory = yash_directory + "/Themes"
    if isdir(themes_directory) != True:
        copy_tree(file_location + "/themes",themes_directory)



import os
from os import system, name
from prompt_toolkit import print_formatted_text, HTML
from prompt_toolkit.styles import Style
import glob
from pathlib import Path
import subprocess
from string import digits, ascii_letters as letters
from pathlib import Path
import os
import shutil

def execute_commands(command):
    try:
        commands = command.split()
        if "cd" == commands[0]:
            if len(commands[1:]) == 0:
                os.chdir(Path.home())
            else:
                _,_,dir = command.partition(' ')
                os.chdir(dir)
            return
        elif "mkdir" == commands[0]:
            path = Path(os.getcwd())
            new_dir = os.path.join(path, commands[1])
            os.mkdir(new_dir)
            return
        elif "rm" == commands[0]:
            if commands[1] == "-rf":
                path = Path(os.getcwd())
                directory = os.path.join(path, commands[2])
                if os.path.exists(directory) and os.path.isdir(directory):
                    shutil.rmtree(directory, ignore_errors=True)
            else:
                os.remove(command[1])
            return


        subprocess.run(commands)
    except Exception:
        print("yash: command not found:", command)

def clear():
    if name == "nt":
        _ = system('cls')
    else:
        _ = system('clear')

def alphanumeric_key(text):
    return [c.lower() for c in text if c in letters + digits]

def ls():
    files = glob.glob("*")
    files.sort(key = alphanumeric_key)
    directory = Style.from_dict({
        'dir': '#0FFFFF bold'
    })
    files_style = Style.from_dict({
        'files': '#FFFFFF bold'
    })
    for file in files:
        if os.path.isdir(file):
            print_formatted_text(HTML("<dir>" + file + "</dir>"), end=" ", style=directory)
        else:
            print_formatted_text(HTML("<files>" + file + "</files>"), end=" ", style=files_style)
    print()

def cd(path):
    ch = ''.join(path)
    try:
        os.chdir(os.path.abspath(ch))
    except Exception as e:
        print("cd: no such file or directory. ", e)



import subprocess
from prompt_toolkit.shortcuts.prompt import prompt
from neosh.neosh.functions import execute_commands, ls, clear
from neosh.checker import checker, yaml_directory
from pathlib import Path
from pygments.lexers.shell import FishShellLexer
from prompt_toolkit.lexers import PygmentsLexer
from prompt_toolkit import PromptSession
from prompt_toolkit.history import FileHistory
from prompt_toolkit.completion import FuzzyCompleter
import sys
from prompt_toolkit.auto_suggest import AutoSuggestFromHistory
import os
from prompt_toolkit.completion import merge_completers, WordCompleter
import glob
import importlib
import yaml

def shell():
    HOME_DIR = str(Path.home())
    env = os.path.expanduser(os.path.expandvars(HOME_DIR + "/.config/neosh/Themes"))
    sys.path.append(env)
    checker()
    yaml_dir = open(yaml_directory, "r")
    yamlContents = yaml.load(yaml_dir, Loader = yaml.FullLoader)
    theme = importlib.import_module(yamlContents["theme"])
    history = Path(HOME_DIR + "/.neosh_history")
    session = PromptSession(history=FileHistory(str(history)))
    
    while True:
        try:
            importlib.reload(theme)
            bash_commands = WordCompleter([
                "ls",
                "cd",
                "clear",
                "cat",
                "python3",
                "python",
                "pip3",
                "pip",
                "exit",
                "neofetch",
                "sudo",
                "su"
            ])
            Files = WordCompleter(glob.glob("*"))
            
            merged_completers = merge_completers([bash_commands, Files])

            command = session.prompt(theme.input_required,
                                     style=theme.style,
                                     lexer=PygmentsLexer(FishShellLexer),
                                     auto_suggest=AutoSuggestFromHistory(),
                                     completer=FuzzyCompleter(merged_completers),
                                     complete_while_typing= False)
            

            if command == "exit":   
                print("exit")
                break
            elif command == "ls":
                ls()
            elif len(command) == 0:
                pass
            elif command == "clear":
                clear()
            elif command == "git commit":
                message = prompt("Enter Commit Message: ")
                subprocess.run("git commit -m \""+ message +"\"")
            elif command == "help":
                print("yash : Yet Another SHell written in python.")
            else:
                try:
                    try:
                        print(eval(command))
                    except:
                        out = exec(command)
                        if out != None:
                            print(out)
                except:
                    try:
                        execute_commands(command)
                    except:
                        os.system(command)
           
            command = command + "\n"


        except KeyboardInterrupt:
            sys.exit("\nexit")

        except EOFError:
            sys.exit("\nexit")



from setuptools import setup
import pathlib
HERE = pathlib.Path(__file__).parent

README = (HERE / "README.md").read_text()

setup(
    name = "neosh",
    packages = ["neosh", "neosh.neosh", "neosh.themes"],
    version = "0.0.1",
    description = "Yet Another SHell written in python",
    long_description = README,
    long_description_content_type = "text/markdown",
    url="https://github.com/neodrags/yash-shell",
    author="Prateek Kesavarapu",
    author_email="kesavarapu.prateek@gmail.com",
    license="MIT",
    include_package_data = True,
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    install_requires=["prompt-toolkit", "pygments"],
    entry_points = {
        "console_scripts": [
            "neosh = neosh.neosh.shell:shell"
        ]
    }
)
